/* Copyright (C) 2005, 2006, 2007, 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package backend;

/**
 * LittleEndian: implementation of little endian access to backend
 * 
 * @author S. Frenz
 * @version 090207 added copyright notice
 *  version 080518 moved createNewInstruction hereto
 *  version 070615 removed no longer needed get*
 *  version 070114 reduced access level where possible
 *  version 061217 bugfixed getLong
 *  version 060607 initial version
 */

public abstract class LittleEndian extends Architecture {
  public void putShort(int pos, short value) {
    mem.putByte(pos++, (byte)value);
    mem.putByte(pos,   (byte)(value>>>8));
  }
  
  public void putInt(int pos, int value) {
    mem.putByte(pos++, (byte)value);
    mem.putByte(pos++, (byte)(value>>>8));
    mem.putByte(pos++, (byte)(value>>>16));
    mem.putByte(pos,   (byte)(value>>>24));
  }
  
  public void putLong(int pos, long value) {
    mem.putByte(pos++, (byte)value);
    mem.putByte(pos++, (byte)(value>>>8));
    mem.putByte(pos++, (byte)(value>>>16));
    mem.putByte(pos++, (byte)(value>>>24));
    mem.putByte(pos++, (byte)(value>>>32));
    mem.putByte(pos++, (byte)(value>>>40));
    mem.putByte(pos++, (byte)(value>>>48));
    mem.putByte(pos,   (byte)(value>>>56));
  }

  public Instruction createNewInstruction() {
    return new LittleEInstr(maxInstrCodeSize);
  }
}
