/* Copyright (C) 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package backend.srb;

import osio.TextPrinter;
import backend.Instruction;
import compbase.Context;
import compbase.Mthd;
import memory.MemoryImage;

/**
 * SRBTarget: interface for SRB optimized code generation methods 
 * 
 * @author S. Frenz
 * @version 090215 added methods required for register resolving, changed ins* to handle* with phase
 *  version 090207 added copyright notice
 *  version 090118 added getRegisterCount
 *  version 081226 initial version
 */

public interface SRBTarget {
  public void putRef(int pos, int ptr);
  public void putCodeRef(int pos, int ptr);
  public void copyMethodCode(int destination);
  public int getMethodSize();
  public void init(MemoryImage imem, int ilev, Context ictx);
  public Instruction getUnlinkedInstruction();
  public void appendInstruction(Instruction i);
  public void finalizeMethodCoding();
  
  public void backInit(SRB srb);
  public boolean hasError();
  public int getRegisterCount(int type);
  
  public SRBNativeRegisterInfo createNativeRegisterInfo();
  public void initSpecialRegister(SRBRegister reg);
  public void resolveRegister(SRBRegister reg);
  public void mapNativeRegister(SRBRegister reg);
  public void printNativeInfo(SRBNativeRegisterInfo info, TextPrinter v);
  
  public void handleMthdFrame(int enter, Mthd refMthd, int phase);
  public void handleFlowHint(int hint, int id, int phase);
  public void handleLoadConstVal(SRBRegister dst, int ival, long lval, int type, int phase);
  public void handleLoadVar(boolean loadValueNotAddress, SRBRegister dst, SRBRegister src, int pos, int type, int phase);
  public void handleDupConv(SRBRegister dst, SRBRegister src, int toType, int fromType, boolean intfMapSpecial, int phase);
  public void handleSingleStackOp(boolean push, SRBRegister reg, int ival, long lval, int type, int phase);
  public void handleAssign(SRBRegister dst, SRBRegister src, int type, int phase);
  public void handleRegOp(SRBRegister dst, SRBRegister src1, SRBRegister src2, int op, int type, int phase);
  public void handleMemOp(SRBRegister dst, int op, int type, int phase);
  public void handleCall(SRBRegister baseReg, int off, int parSize, Mthd mthd, boolean withIntfMap, int phase);
  public void handleJump(Instruction dest, int cond, int phase);
  public void handleCompare(SRBRegister src1, SRBRegister src2, int cond, int val, long lval, int type, int phase);
  public void handleSpecialMemIO(boolean write, SRBRegister dst, SRBRegister src, int type, int memLoc, int phase);
  public void handleDerefAddr(SRBRegister addrReg, SRBRegister indReg, int baseOffset, int entrySize, int phase);
  public void handleCheckBounds(SRBRegister addrReg, SRBRegister offReg, int checkToOffset, Instruction onSuccess, int phase);
  public void handleNativeBoundException(int phase);
  public void handleThrowFrame(int iNr, SRBRegister addrReg, Instruction dest, int throwBlockOffset, int phase);
  public void handleInlineVarOffset(int inlineMode, SRBRegister src, int pos, int baseValue, int phase);
}
