/* Copyright (C) 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package frontend.sjava;

import compbase.Context;
import compbase.FilledParam;
import compbase.Mthd;
import compbase.TypeRef;
import compbase.Unit;

/**
 * ExConstNew: container for "new" that can be done at compile-time, created by ExNew
 * 
 * @author S. Frenz
 * @version 090207 added copyright notice
 *  version 080706 fixed type setting of target
 *  version 080616 initial version
 */

public class ExConstNew extends ExConstInitObj {
  protected ExConstNew nextConstNew; //used to chain constNews in a class
  protected Unit destTypeUnit;
  protected FilledParam arrSizes;
  
  protected ExConstNew(int fid, int il, int ic) {
    super(fid, il, ic);
  }

  public boolean resolve(Unit unitContext, Mthd mthdContext, Context ctx) {
    //should never be called as ExConstNew is called only by ExNew on demand
    compErr(ctx, "invalid call to ExConstnew.resolve");
    return false;
  }

  public boolean fillConstNew(Unit unitContext, Unit dtu,
      TypeRef targetType, FilledParam arrS, Context ctx) {
    JUnit myUnit;
    
    //enter in array-list of unit (has to be done before param-resolve to ensure correct order in list)
    if (!(unitContext instanceof JUnit)) {
      printPos(ctx, "java array initialization in not-java unit is not supported");
      return false;
    }
    target.owner=myUnit=(JUnit)unitContext;
    target.type=targetType;
    destTypeUnit=dtu;
    nextConstNew=myUnit.constNews;
    myUnit.constNews=this;
    arrSizes=arrS;
    getTypeOf(targetType);
    constObject=true;
    return true;
  }
}
