/* Copyright (C) 2005, 2006, 2007, 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package frontend.sjava;

import backend.Instruction;
import compbase.*;

/**
 * ExVal: constant value
 * 
 * @author S. Frenz
 * @version 090207 added copyright notice and replaced genLoadConstNullAddr by genLoadConstVal
 *  version 070909 optimized signature of Expr.resolve
 *  version 070809 added support for float and double
 *  version 070727 replaced exSubResolve by resolve as there is nothing done anymore, optimized genOutputVal
 *  version 070628 adopted changes for "Expr extends TypeRef"
 *  version 070114 reduced access level where possible
 *  version 070111 adapted change in printPos and compErr
 *  version 070106 removed genOutputAddr
 *  version 061229 removed access to firstInstr
 *  version 061203 optimized calls to printPos and compErr
 *  version 061127 optimized genOutputCondJmp
 *  version 060607 initial version
 */

public class ExVal extends Expr {
	protected int intValue; //keeps data for byte, short, char, int, float and boolean
	protected long longValue; //keeps data for long and double
	
	protected ExVal(int fid, int il, int ic) {
		super(fid, il, ic);
	}
	
	public boolean resolve(Unit unitContext, Mthd mthdContext, Context ctx) {
    //nothing to resolve for a constant value
	  return true;
	}
	
	public int calcConstantType(Context ctx) {
    switch (baseType) {
      case T_BYTE: case T_SHRT: case T_INT:
      case T_CHAR: case T_BOOL: case T_FLT:
        return T_INT;
      case T_LONG: case T_DBL:
        return T_LONG;
    }
		return 0;
	}
	
	public int getConstIntValue(Context ctx) {
	  return intValue;
	}
	
	public long getConstLongValue(Context ctx) {
	  return longValue;
	}
	
	public void genOutputVal(int reg, Context ctx) {
    boolean asDouble=false;
    switch (baseType) {
      case T_BYTE:
      case T_SHRT:
      case T_INT:
      case T_CHAR:
      case T_BOOL:
      case T_FLT:
    	  ctx.arch.genLoadConstVal(reg, intValue, baseType);
        return;
      case T_DBL:
        asDouble=true; //no break
    	case T_LONG:
    	  ctx.arch.genLoadConstDoubleOrLongVal(reg, longValue, asDouble);
        return;
      case T_NULL:
        compErr(ctx, "ExVal.genOutputVal with unresolved null-type");
        return;
    	case T_NNPT:
    	  ctx.arch.genLoadConstVal(reg, 0, StdTypes.T_PTR);
        return;
      case T_NDPT:
        ctx.arch.genLoadConstVal(reg, 0, StdTypes.T_DPTR);
        return;
  	}
    compErr(ctx, "ExVal.genOutputVal for unsupported type");
	}	
	
	public void genOutputCondJmp(Instruction jumpDest, boolean isTrue, Instruction elseDest, Context ctx) {
	  if (baseType!=T_BOOL) {
    	compErr(ctx, "ExVal.genOutputJmp with invalid type");
    	return;
	  }
	  if (intValue==(isTrue?1:0)) ctx.arch.genJmp(jumpDest);
	}
}
