/* Copyright (C) 2005, 2006, 2007, 2008, 2009 Stefan Frenz
 * 
 * This file is part of SJC, the Small Java Compiler written by Stefan Frenz.
 * 
 * SJC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SJC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SJC. If not, see <http://www.gnu.org/licenses/>.
 */

package frontend.sjava;

import compbase.*;
import backend.Architecture;
import backend.Instruction;

/**
 * StFor: for-loop
 * 
 * @author S. Frenz
 * @version 090508 adopted changes in Stmt
 *  version 090506 added flow analysis information
 *  version 090207 added copyright notice
 *  version 080610 removed conversion to JMthd as it is no longer required
 *  version 080508 added flow hints
 *  version 070913 made init and lupd of type Stmt, added support for spread local variable declaration 
 *  version 070909 optimized signature of Expr.resolve, fixed position of contDest, adopted changes in StBreakable
 *  version 070908 optimized signature of Stmt.resolve
 *  version 070114 reduced access level where possible
 *  version 070111 adapted change in printPos and compErr
 *  version 061229 removed access to firstInstr
 *  version 061203 optimized calls to printPos and compErr
 *  version 061128 added support for expression lists
 *  version 060607 initial version
 */

public class StFor extends StBreakable {
	protected Stmt init, loStmt, lupd;
	protected Expr cond;
	
	protected StFor(StBreakable io, StringList ila, int fid, int il, int ic) {
		super(io, ila, fid, il, ic);
	}
	
	protected int innerResolve(int flowCode, Unit unitContext, Mthd mthdContext, Context ctx) {
    Stmt jel;
    Vrbl oldVarState;
    
    oldVarState=mthdContext.vars;
    jel=init;
	  while (jel!=null) {
      if ((jel.resolve(0, unitContext, mthdContext, ctx)&FA_ERROR)!=0) return FA_ERROR; //init-stmts may not change flow
      jel=jel.nextStmt;
    }
	  if (cond!=null) if (!cond.resolve(unitContext, mthdContext, ctx)) return FA_ERROR;
    jel=lupd;
	  while (jel!=null) {
      if ((jel.resolve(0, unitContext, mthdContext, ctx)&FA_ERROR)!=0) return FA_ERROR; //update-stmts may not change flow
      jel=jel.nextStmt;
    }
	  if (cond!=null && !cond.isBoolType()) {
	    printPos(ctx, "need boolean type in for-condition");
	    return FA_ERROR;
	  }
    flowCode=loStmt.resolve(flowCode, unitContext, mthdContext, ctx);
    if (cond!=null && cond.calcConstantType(ctx)==StdTypes.T_INT && cond.getConstIntValue(ctx)==1
        && (flowAnalysisBuffer&FA_HAS_ENDBLOCK)==0) { //endless loop without break
      flowCode|=FA_NEXT_IS_UNREACHABLE;
    }
    mthdContext.vars=oldVarState;
	  return flowCode;
	}
	
	protected void genOutput(Context ctx) {
    Stmt jel;
    Instruction loopDest, condDest;
    int id;
    
    loopDest=ctx.arch.getUnlinkedInstruction();
    condDest=ctx.arch.getUnlinkedInstruction();
    contDest=ctx.arch.getUnlinkedInstruction();
	  breakDest=ctx.arch.getUnlinkedInstruction();
    id=ctx.arch.insertFlowHint(Architecture.F_BLOCKSTART, 0);
    jel=init;
	  while (jel!=null) {
      jel.genOutput(ctx);
      jel=jel.nextStmt;
    }
	  ctx.arch.insertFlowHint(Architecture.F_CONDSTART, id);
	  ctx.arch.appendInstruction(condDest);
	  if (cond!=null) cond.genOutputCondJmp(breakDest, false, loopDest, ctx);
    ctx.arch.insertFlowHint(Architecture.F_LOOPSTART, id);
    ctx.arch.appendInstruction(loopDest);
	  loStmt.genOutput(ctx);
    ctx.arch.appendInstruction(contDest);
    jel=lupd;
	  while (jel!=null) {
      jel.genOutput(ctx);
      jel=jel.nextStmt;
    }
	  if (cond!=null) ctx.arch.genJmp(condDest);
    else ctx.arch.genJmp(loopDest);
	  ctx.arch.appendInstruction(breakDest);
    ctx.arch.insertFlowHint(Architecture.F_BLOCKEND, id);
    contDest=breakDest=null;
	}
}
